__author__  = 'Martin Aspeli <optilude@gmx.net>'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *

from Products.PloneJobBoard import permissions
from Products.PloneJobBoard.fields import JobBoardsField
from Products.PloneJobBoard.widgets import JobBoardsWidget

JobPostingSchema = BaseFolderSchema + Schema((
    TextField('description',
        accessor = 'Description',
        required = True,
        searchable = True,
        widget = TextAreaWidget(
            description = """Please enter a short description for the job posting, one or two lines at most.""",
            description_msgid = 'PloneJobBoard_help_jobposting_description',
            i18n_domain = 'PloneJobBoard',
            label = "Short description",
            label_msgid = 'PloneJobBoard_label_jobposting_description',
        ),
    ),
    
    StringField('jobType',
        required = True,
        searchable = True,
        index = ':schema',
        widget = StringWidget(
            description = """Please enter the type of position offered""",
            description_msgid= ' PloneJobBoard_help_jobType',
            i18n_domain = 'PloneJobBoard',
            label = "Job type",
            label_msgid = 'PloneJobBoard_label_jobType',
        ),
    ),
    
    StringField('employmentType',
        required = True,
        searchable = True,
        widget = StringWidget(
            description = """Please enter the type of employment offered(e.g. part time, full time)""",
            description_msgid = 'PloneJobBoard_help_employmentType',
            i18n_domain = 'PloneJobBoard',
            label = "Employment type",
            label_msgid = 'PloneJobBoard_label_employmentType',
        ),
    ),
    
    StringField('jobLocation',
        searchable = True,
        index = ':schema',
        widget = StringWidget(
            description = "Please enter a description of where the job is based",
            description_msgid = 'PloneJobBoard_help_jobLocation',
            i18n_domain = 'PloneJobBoard',
            label = "Location",
            label_msgid = 'PloneJobBoard_label_jobLocation',
        ),
    ),
    
    TextField('jobDescription',
        allowable_content_types = ('text/html', 'text/structured', 'text/restructured', 'text/plain'),
        default_content_type = 'text/structured',
        default_output_type = 'text/html',
        required = True,
        searchable = True,
        widget = RichWidget(
            description = """Please enter a detailed job description""",
            description_msgid = 'PloneJobBoard_help_jobDescription',
            i18n_domain = 'PloneJobBoard',
            label = "Job description",
            label_msgid = 'PloneJobBoard_label_jobDescription',
            rows = 20,
        ),
    ),
    
    LinesField('jobRequirements',
        searchable = True,
        widget = LinesWidget(
            description = """Please enter any specific job requirements, one per line""",
            description_msgid = 'PloneJobBoard_help_jobRequirements',
            i18n_domain = 'PloneJobBoard',
            label = "Job requirements",
            label_msgid = 'PloneJobBoard_label_jobRequirements',
            rows = 10,
        ),
    ),
    
    StringField('contactName',
        widget = StringWidget(
            description = """Please enter a contact name""",
            description_msgid = 'PloneJobBoard_help_contactName',
            i18n_domain = 'PloneJobBoard',
            label = "Contact name",
            label_msgid = 'PloneJobBoard_label_contactName',
        ),
    ),
    
    StringField('contactEmail',
        validators = ('isEmail',),
        widget = StringWidget(
            description = """Please enter a contact email address""",
            description_msgid = 'PloneJobBoard_help_contactEmail',
            i18n_domain = 'PloneJobBoard',
            label = "Contact email address",
            label_msgid = 'PloneJobBoard_label_contactEmail',
        ),
    ),
    
    StringField('contactPhone',
        widget = StringWidget(
            description = """Please enter a contact telephone number""",
            description_msgid = 'PloneJobBoard_help_contactPhone',
            i18n_domain = 'PloneJobBoard',
            label = "Contact telephone number",
            label_msgid = 'PloneJobBoard_label_contactPhone',
        ),
    ),
    
    TextField('contactAddress',
        widget = TextAreaWidget(
            description = """Please enter a contact address""",
            description_msgid = 'PloneJobBoard_help_contactAddress',
            i18n_domain = 'PloneJobBoard',
            label = "Contact address",
            label_msgid = 'PloneJobBoard_label_contactAddress',
            rows = 7,
        ),
    ),
    
    TextField('extraInformation',
        searchable = True,
        widget = TextAreaWidget(
            description = "Please enter any additional information to attach to the job posting",
            description_msgid = 'PloneJobBoard_help_extraInformation',
            i18n_domain = 'PloneJobBoard',
            label = "Extra information",
            label_msgid = 'PloneJobBoard_label_extraInformation',
            rows = 5,
        ),
    ),
    
    JobBoardsField('jobBoards',
        allowed_types = ("JobBoard",),
        required = True,
        relationship = "JobPosting",
        validators = ('isValidJobBoardSelection',),
        widget = JobBoardsWidget(
            categorySeparator = ' // ',
            description = """Please select the job boards you wish the posting to be shown on""",
            description_msgid = 'PloneJobBoard_help_jobBoards',
            i18n_domain = 'PloneJobBoard',
            label = "Job boards",
            label_msgid = 'PloneJobBoard_label_jobBoards',
        ),
    ),
    
    IntegerField('activeDays',
        default_method = 'getActiveDaysDefault',
        widget = IntegerWidget(
            description = """Please enter the number of days the posting will be active for after it has been published.""",
            description_msgid = 'PloneJobBoard_help_activeDays',
            i18n_domain = 'PloneJobBoard',
            label = "Active for",
            label_msgid = 'PloneJobBoard_label_activeDays',
        ),
    ),
    
    ComputedField('expirationDate',
        expression = 'context.calculateExpirationDate()',
        widget = ComputedWidget(
            description = """Expiration date, calculated from the publication/effective date and the the number of active days.""",
            description_msgid = 'PloneJobBoard_help_expirationDate',
            i18n_domain = 'PloneJobBoard',
            label = "Expiration date",
            label_msgid = 'PloneJobBoard_label_expirationDate',
            modes = ('View',),
            visible = {'view' : 'visible', 'edit' : 'invisible'},
        ),
    ),
    
))

class JobPosting(BaseFolder):
    """A job posting, which may appear on a job board. Members create a
    job posting, and select which board(s) it will be displayed on.
    Responses are stored as JobResponse objects inside, which are
    visible only to the JobPosting's creator.
    """
    security = ClassSecurityInfo()
    
    portal_type = meta_type = 'JobPosting' 
    archetype_name = 'Job posting'
    allowed_content_types = ('JobResponse',)
    filter_content_types = True
    allow_discussion = False
    content_icon = 'JobPosting.gif'
    global_allow = True

    _at_rename_after_creation = True

    actions =  (
       {'action':      "string:${object_url}",
        'category':    "object",
        'id':          'view',
        'name':        'View',
        'permissions': (permissions.View,),
        'condition'  : 'python:1'
       },
       {'action':      "string:${object_url}/edit",
        'category':    "object",
        'id':          'edit',
        'name':        'Edit',
        'permissions': (permissions.ModifyPortalContent,),
        'condition'  : 'python:1'
       },
    )

    aliases = {
        '(Default)'  : 'jobposting_view',
        'view'       : 'jobposting_view',
        'edit'       : 'base_edit',
    }
    
    schema = JobPostingSchema

    security.declareProtected(permissions.View, 'selectedBoards')
    def selectedBoards(self):
        """Get a list of UIDs for the selected job boards. Indexed in the
        catalogue to enable efficient display of postings on job boards.
        """
        return self.getRawJobBoards()


    security.declareProtected(permissions.View, 'getNotAddableTypes')
    def getNotAddableTypes(self):
        """To ensure that users aren't allowed to add a JobResponse when
        they already have added one, do a test to see whether the
        current authenticated member has added a response, and if so
        return JobResponse as a not-addable type.
        """
        if self.canRespond():
            return ()
        else:
            return ('JobResponse',)

    security.declareProtected(permissions.View, 'getActiveDaysDefault')
    def getActiveDaysDefault(self):
        """Get the default value for the active days field. This is set in
        the property portal_properties/site_properties/jobPostingDefaultDays, 
        and default to 30 days if this is not not found.
        """
        ptool = getToolByName(self, 'portal_properties')
        site_properties = getattr(ptool, 'site_properties')
        return site_properties.getProperty('jobPostingDefaultDays', 30)


    security.declareProtected(permissions.ModifyPortalContent, 'validate_activeDays')
    def validate_activeDays(self, value):
        """Ensure that the number of active days does not exceed a maximum
        set either in portal_properties/site_properties/jobPostingMaxDays or, if 
        that does not exist, a default of 30 days. If the above property
        exists and is set to 0, permit any lifespan.
        """
        try:
            value = int(value)
        except ValueError:
            return "You must enter a positive whole number"
        
        maxDays = self.getActiveDaysDefault()

        if maxDays <= 0:
            return None
        elif int(value) > int(maxDays):
            return "You may not make a job posting active for more than %s days" % (maxDays)

    security.declareProtected(permissions.View, 'calculateExpirationDate')
    def calculateExpirationDate(self):
        """Calculate the expiration date of the posting, by adding the
        number of active days to the effective date. Returns None if the
        effective date is not set.
        """
        if self.getEffectiveDate():
            return self.getEffectiveDate() + self.getActiveDays()
        else:
            return None

    
    security.declareProtected(permissions.View, 'isOpen')
    def isOpen(self):
        """Return true if this posting is generally open for responses
        """
        # We can't respond if the object has expired
        if self.contentExpired():
            return False
        
        wftool = getToolByName(self, 'portal_workflow')
        return wftool.getInfoFor(self, 'review_state') == 'published'

    def isOwner(self):
        """Return true if this posting is owned by the current member
        """
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        
        if member.getUserName() in self.Creators():
            return True
        else:
            return False
    
    def alreadyResponded(self):
        """Return true if the current member has already responded
        """
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        
        catalog = getToolByName(self, 'portal_catalog')
        # Need to use unrestricted since the authenticated member doesn't have
        # permission to see the posting :)
        contents = catalog.unrestrictedSearchResults(path = '/'.join(self.getPhysicalPath()),
                                                     portal_type = 'JobResponse')
        
        for response in contents:
            if response.Creator == member.getUserName():
                return True
        return False
        
    def canRespond(self):
        membership = getToolByName(self, 'portal_membership')
        return self.isOpen() and not self.isOwner() and \
                not self.alreadyResponded() and not membership.isAnonymousUser()
        
registerType(JobPosting)


